jQuery(document).ready(function ($) {

    /**
     * TODO Actions
     *
     * @type {{VolumeChange: [], DesLike: [], Play: [], Pause: [], Like: [], PlayerMouseEnter: [], PlayerMouseMove: [], CommentSend: [], PlayerMouseLeave: []}}
     */
    window.VPP_PLAYER_ACTIONS = {
        Generate: [],
        Play: [],
        Pause: [],
        VideoInterval: [],
        VideoEnd: [],
        PlayerMouseEnter: [],
        PlayerMouseLeave: [],
        PlayerMouseMove: [],
        ScreenTimeOut: [],
        FullScreen: [],
        ProgressChange: [],
        ProgressMouseUp: [],
        VolumeChange: [],
    };

    /**
     * TODO Add Action
     *
     * @param ACTION
     * @param CALLBACK
     */
    const AddAction = function (ACTION, CALLBACK) {

        window.VPP_PLAYER_ACTIONS[ACTION].push(CALLBACK);

    }

    /**
     * TODO Do Action
     *
     * @param ACTION
     * @param ARG
     * @constructor
     */
    const DoAction = function (ACTION, ARG = null) {

        $.each(window.VPP_PLAYER_ACTIONS[ACTION], function (Index, Callback) {

            Callback(ARG);

        })

    }

    // jQuery $.fn
    $.fn.VPP_GENERATE = function () {

        VPP_GENERATE();

    }

    // Vpp class generate
    $.fn.VPP_CLASS_GENERATE = function (CLASS) {

        let This = $(this);

        if (typeof CLASS === 'string') return false;

        $.each(Class, function (Index, Value) {

            if ($.inArray(Value, CLASS) !== -1) {

                This.addClass(Value);

            } else {

                This.removeClass(Value);

            }

        })

    }

    // Storage of elements and events and functions
    const Stack = {}

    // Classes change in different modes and are stored in the same resource
    const Class = {
        VideoPlayed: 'ytpl-video-played',
        Thumbnail: 'ytpl-thumbnail',
        Overlay: 'ytpl-overlay',
        DarkOverlay: 'ytpl-dark-overlay',
        TopBar: 'ytpl-top-bar',
        Controller: 'ytpl-controller',
        VolumeBar: 'ytpl-volume-bar',
        SettingList: 'ytpl-setting',
        SettingListSetting: 'setting',
        SettingListQuality: 'quality',
        SettingListSpeed: 'speed',
        SettingListSubtitle: 'subtitle',
        MoreVideo: 'ytpl-more',
        MoreVideoButton: 'ytpl-more-button',
        EndPoint: 'ytpl-end-point',
        EndScreen: 'ytpl-end-screen',
        Similar: 'ytpl-similar',
        Description: 'ytpl-description',
        CartNotification: 'ytpl-cart-notification',
        CartBox: 'ytpl-cart-box',
        IsFullScreen: 'ytpl-is-full-screen',
        Share: 'ytpl-share',
        Info: 'ytpl-info',
        Comment: 'ytpl-comment',
        Subtitle: 'ytpl-subtitle',
        Context: 'ytpl-context-menu',
        Loading: 'ytpl-loading',
        Message: 'ytpl-message',
    }

    // Window loaded
    $(window).on('load', function () {

        VPP_GENERATE();

    });

    // Set element on stack and add events
    const VPP_GENERATE = function () {

        $.each($('.ytpl-render'), function () {

            // show player
            $(this).show();

            let This = $(this);
            let ID = This.attr('id');
            let Player = This.find('.ytpl-player');
            let Playlist = This.find('.ytpl-playlist');
            let Controller = Player.find('.ytpl-controller');
            let Progress = Controller.find('.ytpl-progress');
            let VolumeProgress = Controller.find('.ytpl-volume-process');
            let TopBar = Player.find('.ytpl-top-bar');
            let Context = Player.find('.ytpl-context-menu');

            // Preview
            let Preview = Progress.find('.ytpl-preview');
            let Preview_Canvas = Preview.find('canvas');

            if (Stack[ID]) return true;

            // Set data
            Stack[ID] = {

                // Basic
                ID: ID,
                Render: This,
                Player: {
                    This: Player,
                    Status: false,
                    IsPlayed: false,
                    IsFullScreen: false,
                    Video: Player.find('.ytpl-video'),
                    VideoContainer: Player.find('.ytpl-video-player'),
                    Preview: {
                        This: Preview,
                        Status: false,
                        Video: document.createElement('video'),
                        FnWidth: (Size) => Preview_Canvas.width(Size),
                        FnHeight: (Size) => Preview_Canvas.height(Size),
                        FnTime: (Time) => Progress.find('.ytpl-preview span').html(TO_HHMMSS(Time.toString())),
                        FnCanvas: (Object, X, Y, Width, Height) => {

                            let Context = Preview_Canvas[0].getContext('2d');
                            Context.drawImage(Object, X, Y, Width, Height);

                        },
                    },
                    Controller: {
                        This: Controller,
                        Status: false,
                        Play: Controller.find('.ytpl-play'),
                        Pause: Controller.find('.ytpl-pause'),
                        FullScreen: Controller.find('.ytpl-full-screen'),
                        Setting: Controller.find('.ytpl-setting'),
                        Volume: {
                            This: Controller.find('.ytpl-volume'),
                            Progress: VolumeProgress,
                            Range: VolumeProgress.find('input'),
                            Point: VolumeProgress.find('.ytpl-point-hover'),
                            Progressed: VolumeProgress.find('.ytpl-processed'),
                        },
                        Progress: {
                            This: Progress,
                            Range: Progress.find('input'),
                            Point: Progress.find('.ytpl-point-hover'),
                            Progressed: Progress.find('.ytpl-progressed'),
                        },
                        Popup: {
                            Setting: Controller.find('.ytpl-setting-popup.setting'),
                            Speed: Controller.find('.ytpl-setting-popup.speed'),
                        },
                        FnTime: (Time) => Controller.find('.ytpl-current').html(TO_HHMMSS(Time.toString())),
                    },
                    TopBar: {
                        This: TopBar,
                        Status: false,
                        Description: TopBar.find('.ytpl-description-btn'),
                        Info: {
                            Status: false,
                            Button: TopBar.find('.info'),
                            Content: This.find('.ytpl-info'),
                        },
                        Share: {
                            Button: TopBar.find('.share'),
                            Content: This.find('.ytpl-share'),
                        },
                    },
                    Context: {
                        This: Context,
                        Status: false,
                    },
                    Config: JSON.parse(Player.attr('data-config')),
                    Interval: {
                        Video: null,
                        Screen: null,
                        Preview: null,
                        Subtitle: null,
                    },
                },

            };

            Stack[ID].Player.Preview.Video.src = Stack[ID].Player.Config.src;
            Player.removeAttr('data-config');

            if (Playlist[0]) {

                Stack[ID].Playlist.Config = JSON.parse(Playlist.attr('data-config'));
                Playlist.removeAttr('data-config');

            }

            // Do action `Generate`
            DoAction('Generate', Stack[ID]);

            RATIO_DETECT(ID);
            Event(ID);

        });

    }

    // TODO Event
    const Event = function (ID) {

        // Mouse enter
        Stack[ID].Player.This.on('mouseenter', () => MouseEnter(ID));

        // Mouse leave
        Stack[ID].Player.This.on('mouseleave', () => MouseLeave(ID));

        // Mouse move
        Stack[ID].Player.This.on('mousemove', () => MouseMove(ID));

        // Double click
        Stack[ID].Player.This.on('dblclick', () => FullScreen(ID));

        // Context menu
        Stack[ID].Player.This.on('contextmenu', function (e) {

            e.stopPropagation();

            ContextMenu(ID, this, e)

            return false;

        });

        // Play
        $.each([Stack[ID].Player.Video, Stack[ID].Player.Controller.Play], function () {
            this.on('click', () => Play(ID));
        });

        // Pause
        Stack[ID].Player.Controller.Pause.on('click', () => Pause(ID));

        // Progress change
        Stack[ID].Player.Controller.Progress.Range.on('input', () => ProgressChange(ID));

        // Progress mouse up
        Stack[ID].Player.Controller.Progress.Range.on('input', () => ProgressMouseUp(ID));

        // Full Screen
        Stack[ID].Player.Controller.FullScreen.on('click', () => FullScreen(ID));

        // Preview
        Stack[ID].Player.Controller.Progress.This.on('mousemove', (e) => Preview(ID, e));

        // Volume mouse enter
        $.each([Stack[ID].Player.Controller.Volume.This, Stack[ID].Player.Controller.Volume.Progress], function () {

            this.on('mouseenter', () => Stack[ID].Player.This.addClass(Class.VolumeBar));

            this.on('mouseleave', () => Stack[ID].Player.This.removeClass(Class.VolumeBar));

        })

        // Volume change
        Stack[ID].Player.Controller.Volume.Range.on('input', () => VolumeChange(ID));

        // Context Item
        Stack[ID].Player.Context.This.find('li').on('click', function () {
            ContextItem(ID, this);
        });

        // Open info
        Stack[ID].Player.TopBar.Info.Button.on('click', () => {

            Stack[ID].Player.TopBar.Info.Status = true;
            Stack[ID].Player.This.addClass(Class.Info)

        });

        // Close info
        Stack[ID].Player.This.find('.ytpl-info .ytpl-close').on('click', () => {

            Stack[ID].Player.TopBar.Info.Status = false;
            Stack[ID].Player.This.removeClass(Class.Info);

        });

        // Toggle description
        Stack[ID].Player.TopBar.Description.on('click', () => Description(ID));

        // Toggle description
        Stack[ID].Player.Controller.Setting.on('click', () => Stack[ID].Player.This.addClass([Class.SettingList, Class.SettingListSetting]));

        // Popup setting item
        Stack[ID].Player.Controller.Popup.Setting.find('li').on('click', function () {
            PopupSettingItem(ID, this)
        });

        // Popup speed item
        Stack[ID].Player.Controller.Popup.Speed.find('li').on('click', function () {
            PopupSpeedItem(ID, this)
        });

        // Document bind - mouse down
        $(document).bind("mousedown", function (e) {

            // check context menu
            if (!$(e.target).parents(".ytpl-context-menu").length > 0) {

                // Hide it
                Stack[ID].Player.This.removeClass(Class.Context);

            }

            // check popup menu
            if (!$(e.target).parents(".ytpl-setting-popup").length > 0) {

                // Hide it
                Stack[ID].Player.This.removeClass([
                    Class.SettingList,
                    Class.SettingListSetting,
                    Class.SettingListSpeed,
                ]);

            }

        });

    }

    // TODO Mouse Enter
    const MouseEnter = function (ID) {

        // Stop leave
        if (
            Stack[ID].Player.This.hasClass(Class.Loading) ||
            Stack[ID].Player.This.hasClass(Class.Info)
        ) return false;

        // Do action `PlayerMouseEnter`
        DoAction('PlayerMouseEnter', Stack[ID]);

    }

    // TODO Mouse Leave
    const MouseLeave = function (ID) {

        // Stop leave
        if (
            Stack[ID].Player.This.hasClass(Class.Loading) ||
            Stack[ID].Player.This.hasClass(Class.Info)
        ) return false;

        if (Stack[ID].Player.IsPlayed) {

            Stack[ID].Player.This.VPP_CLASS_GENERATE([
                Class.VideoPlayed,
            ]);

        }

        // Do action `PlayerMouseLeave`
        DoAction('PlayerMouseLeave', Stack[ID]);

    }

    // TODO Mouse Move
    const MouseMove = function (ID) {

        SCREEN_TIMEOUT(ID);

        // Do action `PlayerMouseMove`
        DoAction('PlayerMouseMove', Stack[ID]);

    }

    // TODO Play Video
    const Play = function (ID) {

        // Pause video
        if (Stack[ID].Player.IsPlayed) {

            Pause(ID);

            return false;

        }

        Stack[ID].Player.IsPlayed = true;
        Stack[ID].Player.Video[0].play();
        Stack[ID].Player.This.VPP_CLASS_GENERATE([
            Class.VideoPlayed,
            Class.Overlay,
            Class.Controller,
        ]);

        SCREEN_TIMEOUT(ID);

        // Interval
        Stack[ID].Player.Interval.Video = setInterval(() => {

            let Time = Stack[ID].Player.Video[0].currentTime;
            let Progressed = Time * 100 / Stack[ID].Player.Video[0].duration;

            Stack[ID].Player.Controller.Progress.Range.val(Progressed);
            Stack[ID].Player.Controller.Progress.Point.css({'left': Progressed + '%'});
            Stack[ID].Player.Controller.Progress.Progressed.css({'width': Progressed + '%'});
            Stack[ID].Player.Controller.FnTime(Math.round(Time));

            // Buffer
            Stack[ID].Player.Controller.Progress.This.find('.ytpl-preload').remove();
            for (let Buffer = 0; Buffer < Stack[ID].Player.Video[0].buffered.length; Buffer++) {

                let Start = Stack[ID].Player.Video[0].buffered.start(Buffer);
                let End = Stack[ID].Player.Video[0].buffered.end(Buffer);
                let Duration = Stack[ID].Player.Video[0].duration;

                if (End - 5 < Start) continue;

                Stack[ID].Player.Controller.Progress.This.find('.ytpl-progress-container').append('<div class="ytpl-preload" style="width:' + (((End - Start) * 100) / Duration) + '%; left:' + ((Start * 100) / Duration) + '%;"></div>');

            }

            // Do action `VideoInterval`
            DoAction('VideoInterval', Stack[ID]);

            // End video
            if (Stack[ID].Player.Video[0].ended) {

                clearInterval(Stack[ID].Player.Interval.Video)
                Stack[ID].Player.IsPlayed = false;
                Stack[ID].Player.Video[0].pause();

                Stack[ID].Player.This.VPP_CLASS_GENERATE([
                    Class.Overlay,
                    Class.TopBar,
                    Class.Controller,
                ]);

                // Do action `VideoEnd`
                DoAction('VideoEnd', Stack[ID]);

            }

        }, 1000)

        // Do action `Play`
        DoAction('Play', Stack[ID]);

    }

    // TODO Pause Video
    const Pause = function (ID) {

        clearInterval(Stack[ID].Player.Interval.Video)
        Stack[ID].Player.IsPlayed = false;
        Stack[ID].Player.Video[0].pause();

        Stack[ID].Player.This.VPP_CLASS_GENERATE([
            Class.Overlay,
            Class.TopBar,
            Class.Controller,
        ]);

        // Do action `Pause`
        DoAction('Pause', Stack[ID]);

    }

    // TODO Progress Change
    const ProgressChange = function (ID) {

        let This = Stack[ID].Player.Controller.Progress.Range;
        let Percent = This.val();
        let Duration = Stack[ID].Player.Video[0].duration;
        let Time = (parseFloat(Percent) * Duration / 100);

        Stack[ID].Player.Video[0].currentTime = Time;
        Stack[ID].Player.Controller.Progress.Point.css({'left': Percent + '%'});
        Stack[ID].Player.Controller.Progress.Progressed.css({'width': Percent + '%'});
        Stack[ID].Player.Controller.FnTime(Math.round(Time));

        Stack[ID].Player.This.VPP_CLASS_GENERATE([
            Stack[ID].Player.IsPlayed ? Class.VideoPlayed : null,
            !Stack[ID].Player.IsPlayed ? Class.TopBar : null,
            Class.Controller
        ]);

        // Do action `ProgressChange`
        DoAction('ProgressChange', Stack[ID]);

    }

    // TODO Progress Mouse Up
    const ProgressMouseUp = function (ID) {

        // Do action `ProgressMouseUp`
        DoAction('ProgressMouseUp', Stack[ID]);

    }

    // TODO Full Screen
    const FullScreen = function (ID) {

        let Screen = Stack[ID].Player.VideoContainer[0];

        try {

            if (!document.fullscreenElement) {

                Stack[ID].Player.IsFullScreen = true;
                Stack[ID].Player.This.addClass(Class.IsFullScreen);
                if (Screen.requestFullscreen) Screen.requestFullscreen({navigationUI: "show"});
                else if (Screen.mozRequestFullScreen) Screen.mozRequestFullScreen();
                else if (Screen.webkitRequestFullscreen) Screen.webkitRequestFullscreen();
                else if (Screen.msRequestFullscreen) Screen.msRequestFullscreen();

                screen.orientation.lock("landscape")
                    .then(function () {
                        console.log('Locked');
                    })
                    .catch(function (error) {
                        console.log(error);
                    });

            } else {

                Stack[ID].Player.IsFullScreen = false;
                Stack[ID].Player.This.removeClass(Class.IsFullScreen);
                if (document.exitFullscreen) document.exitFullscreen();
                else if (document.mozCancelFullScreen) document.mozCancelFullScreen();
                else if (document.webkitExitFullscreen) document.webkitExitFullscreen();
                else if (document.msExitFullscreen) document.msExitFullscreen();

            }


        } catch (err) {

            console.log('Dev Error: ' + err.message);

        }

        // Do action `FullScreen`
        DoAction('FullScreen', Stack[ID]);

    }

    // TODO Preview
    const Preview = function (ID, e) {

        let This = Stack[ID].Player.Controller.Progress.This;
        let Percent = e.offsetX * 100 / This.width();
        let Video = Stack[ID].Player.Video[0];
        let VideoDuration = Video.duration;
        let VideoCurrentTime = Percent * VideoDuration / 100;
        let PreviewWidth = Stack[ID].Player.Preview.This.width();

        Stack[ID].Player.Preview.FnTime(Percent * VideoDuration / 100);
        Stack[ID].Player.Preview.This.css({
            'left': (e.offsetX > (PreviewWidth / 2) ? (e.offsetX + (PreviewWidth / 2) < This.width()) ? e.offsetX - (PreviewWidth / 2) : This.width() - PreviewWidth : 0) + 'px'
        });

        clearTimeout(Stack[ID].Player.Interval.Preview);
        Stack[ID].Player.Interval.Preview = setTimeout(function () {

            Stack[ID].Player.Preview.Video.currentTime = VideoCurrentTime;
            Stack[ID].Player.Preview.Video.addEventListener('seeked', function () {

                let vw = Stack[ID].Player.Preview.Video.videoWidth;
                let vh = Stack[ID].Player.Preview.Video.videoHeight;
                let gcd = RATIO(vw, vh);
                let clw = 94 * ((vw / gcd) / (vh / gcd));
                let canvas = Stack[ID].Player.Preview.This.find('canvas')[0];

                Stack[ID].Player.Preview.FnWidth(clw);
                Stack[ID].Player.Preview.FnHeight(94);
                Stack[ID].Player.Preview.FnCanvas(Stack[ID].Player.Preview.Video, 0, 0, canvas.width, canvas.height);

            });

        }, 80)

    }

    // TODO Volume Change
    const VolumeChange = function (ID) {

        let This = Stack[ID].Player.Controller.Volume.Range;

        Stack[ID].Player.Controller.Volume.Progressed.css({'width': This.val() + '%'})
        Stack[ID].Player.Controller.Volume.Point.css({'left': This.val() + '%'});
        Stack[ID].Player.Video[0].volume = This.val() / 100;

    }

    // TODO Context Menu
    const ContextMenu = function (ID, This, e) {

        let Context = Stack[ID].Player.Context.This;
        let Rect = This.getBoundingClientRect();
        let X = e.clientX - Rect.x;
        let Y = e.clientY - Rect.y;
        let ThisW = This.offsetWidth;
        let ThisH = This.offsetHeight;
        let ContextWidth = Context.width();
        let ContextHeight = Context.height();

        Context.css({
            'top': ((Y + ContextHeight > ThisH) ? (ThisH - ContextHeight - 20) : Y) + 'px',
            'left': ((X + ContextWidth > ThisW) ? (ThisW - ContextWidth - 10) : X) + 'px',
        })
        Stack[ID].Player.This.addClass(Class.Context);

        return false;

    }

    // TODO Context Item
    const ContextItem = function (ID, This) {

        This = $(This);
        let Type = This.attr('data-type');

        switch (Type) {

            case 'loop':

                if (This.hasClass('active')) {
                    This.removeClass('active');
                    Stack[ID].Player.Video[0].loop = false;
                } else {
                    This.addClass('active');
                    Stack[ID].Player.Video[0].loop = true;
                }

                break;

            case 'video_info':

                Stack[ID].Player.TopBar.Info.Status = true;
                Stack[ID].Player.This.addClass(Class.Info);

                break;

        }

        Stack[ID].Player.This.removeClass(Class.Context);

    }

    // TODO Description
    const Description = function (ID) {

        if (Stack[ID].Player.This.hasClass(Class.Description)) {

            Stack[ID].Player.This.removeClass([
                Class.DarkOverlay,
                Class.Description
            ]);

            return false;

        }

        Stack[ID].Player.This.addClass([
            Class.DarkOverlay,
            Class.Description
        ]);

    }

    // TODO Popup Setting Item
    const PopupSettingItem = function (ID, This) {

        let Action = $(This).attr('data-action');

        Stack[ID].Player.This.removeClass([
            Class.SettingListSpeed,
            Class.SettingListSetting,
        ])

        switch (Action) {

            case 'speed':

                Stack[ID].Player.This.addClass(Class.SettingListSpeed);

                break;

        }

    }

    // TODO Popup Speed Item
    const PopupSpeedItem = function (ID, This) {

        let Speed = $(This).attr('value');

        $(This).parent().find('li').removeClass('active');
        $(This).addClass('active');
        Stack[ID].Player.Video[0].playbackRate = Speed;
        Stack[ID].Player.Controller.Popup.Setting.find('li[data-action=speed] div:last-child span').html($(This).html());

    }

    // Control the movement of the mouse on the screen
    const SCREEN_TIMEOUT = (ID) => {

        // Reset cursor
        Stack[ID].Player.This.css({'cursor': 'default'});

        // Interval
        if (Stack[ID].Player.IsPlayed) {

            Stack[ID].Player.This.VPP_CLASS_GENERATE([
                Class.VideoPlayed,
                Class.Overlay,
                Class.Controller,
                Stack[ID].Player.This.hasClass(Class.VolumeBar) ? Class.VolumeBar : null,
                Stack[ID].Player.This.hasClass(Class.Context) ? Class.Context : null,
                Stack[ID].Player.This.hasClass(Class.Info) ? Class.Info : null,
                Stack[ID].Player.This.hasClass(Class.SettingList) ? Class.SettingList : null,
                Stack[ID].Player.This.hasClass(Class.SettingListSetting) ? Class.SettingListSetting : null,
                Stack[ID].Player.This.hasClass(Class.SettingListSpeed) ? Class.SettingListSpeed : null,
            ]);

            clearTimeout(Stack[ID].Player.Interval.Screen)
            Stack[ID].Player.Interval.Screen = setTimeout(function () {

                if (!Stack[ID].Player.IsPlayed) return false;

                Stack[ID].Player.This.css({'cursor': 'none'});
                Stack[ID].Player.This.VPP_CLASS_GENERATE([
                    Class.VideoPlayed,
                    Stack[ID].Player.TopBar.Info.Status ? Class.Info : null
                ]);

                // Do action `ScreenTimeOut`
                DoAction('ScreenTimeOut', Stack[ID]);

            }, 5000)

        }

    }

    // To HH:MM:SS
    const TO_HHMMSS = (Number) => {

        let sec_num = parseInt(Number, 10);
        let hours = Math.floor(sec_num / 3600);
        let minutes = Math.floor((sec_num - (hours * 3600)) / 60);
        let seconds = sec_num - (hours * 3600) - (minutes * 60);

        if (hours < 10) hours = "0" + hours;
        if (minutes < 10) minutes = "0" + minutes;
        if (seconds < 10) seconds = "0" + seconds;

        return (hours > 0 ? hours + ':' : '') + minutes + ':' + seconds;

    }

    // Specify the aspect ratio
    const RATIO = (Width, Height) => {

        if (Height === 0)
            return Width

        return RATIO(Height, Width % Height)

    }

    // Ratio detect
    const RATIO_DETECT = function (ID, Listener = true) {

        let Width = Stack[ID].Player.This.width();
        let Height = Width / (16 / 9);

        Stack[ID].Player.VideoContainer.css({
            'width': Width + 'px',
            'height': Height + 'px',
            'min-height': Height + 'px',
        });

        try {

            // Playlist
            if (Stack[ID].Playlist.This[0]) Stack[ID].Playlist.This.css({'height': Height + 'px'});

        } catch (e) {

            console.log(e)

        }

        // Window resize
        if (Listener) {
            window.addEventListener('resize', function () {

                RATIO_DETECT(ID, false);

            })
        }

    }

});