<?php


class VPP_VIDEO {

	public $ID = null;
	public $Title = null;
	public $Description = null;
	public $Thumbnail = null;
	public $Video = null;
	public $Visibility = null;
	public $User = null;
	public $Time = null;
	public $Modify = null;

	private $Current_Page = 1;
	private $Per_Page = 10;
	private $Order = 'id';
	private $Order_By = 'DESC';
	private $Table = 'vpp_video';

	/**
	 * VPP_VIDEO constructor.
	 *
	 * @param int|null $id
	 *
	 * @version 1.0.0
	 *
	 */
	public function __construct( $id = null ) {

		if ( ! empty( $id ) ) {

			$this->ID = $id;
			$this->Select();

		}

	}

	/**
	 * Config Class
	 *
	 *      [
	 *          current_page: Int (Default: 1)
	 *          per_page: Int (Default: 10)
	 *          order: String (Default: id)
	 *          order_by: String (Default: DESC)
	 *      ]
	 *
	 * @param $arg
	 *
	 * @version 1.0.2
	 *
	 */
	public function Config( $arg ) {

		foreach ( $arg as $index => $value ) {

			switch ( $index ) {

				case 'current_page':
					$this->Current_Page = $value;
					break;

				case 'per_page':
					$this->Per_Page = $value;
					break;

				case 'order':
					$this->Order = $value;
					break;

				case 'order_by':
					$this->Order_By = $value;
					break;

			}

		}

	}

	/**
	 * Select video by id
	 *
	 * @return array|mixed
	 * @version 1.0.0
	 *
	 */
	public function Select() {

		$video = VPP_DATABASE::Select(
			$this->Table,
			[
				'where' => VPP_DATABASE::Where(
					[
						'condition' => [
							[
								'column' => 'id',
								'value'  => $this->ID
							]
						]
					]
				)
			]
		);

		if ( ! empty( $video ) ) {

			$this->SetVariable( $video[0] );

			return $video[0];

		}

		$this->ID = null;

		return [];

	}

	/**
	 * Select more
	 *
	 * @return array|mixed
	 * @version 1.0.0
	 *
	 */
	public function SelectMore() {

		return VPP_DATABASE::Select(
			$this->Table,
			[
				'order' => VPP_DATABASE::Order(
					[
						$this->Order => $this->Order_By != 'DESC'
					]
				),
				'limit' => VPP_DATABASE::Limit( $this->Per_Page, $this->Current_Page )
			]
		);

	}

	/**
	 * Search
	 *
	 * @param $Search
	 *
	 * @return array|mixed
	 * @version 1.0.0
	 *
	 */
	public function Search( $Search ) {

		return VPP_DATABASE::Select(
			$this->Table,
			[
				'limit' => VPP_DATABASE::Limit( 5, 1 ),
				'order' => VPP_DATABASE::Order(
					[
						'id' => false,
					]
				),
				'where' => VPP_DATABASE::Where(
					[
						'condition' => [
							[
								'column'   => 'title',
								'value'    => '%' . $Search . '%',
								'operator' => 'LIKE'
							]
						]
					]
				)
			]
		);

	}

	/**
	 * Insert
	 *
	 * @return bool|int
	 * @version 1.0.0
	 *
	 */
	public function Insert() {

		$result = VPP_DATABASE::Insert(
			$this->Table,
			[
				'title'       => $this->Title,
				'description' => $this->Description,
				'thumbnail'   => $this->Thumbnail,
				'video'       => $this->Video,
				'visibility'  => $this->Visibility,
				'user'        => get_current_user_id(),
				'time'        => time()
			]
		);

		if ( ! empty( $result ) ) {

			$this->ID = $result;

			return $result;

		}

		return false;

	}

	/**
	 * Update
	 *
	 * @return bool|int
	 * @version 1.0.0
	 *
	 */
	public function Update() {

		if ( empty( $this->ID ) ) {
			return false;
		}

		return VPP_DATABASE::Update(
			$this->Table,
			[
				'title'       => $this->Title,
				'description' => $this->Description,
				'thumbnail'   => $this->Thumbnail,
				'video'       => $this->Video,
				'visibility'  => $this->Visibility,
				'modify'      => time()
			],
			VPP_DATABASE::Where(
				[
					'condition' => [
						[
							'column' => 'id',
							'value'  => $this->ID
						]
					]
				]
			)
		);

	}

	/**
	 * Delete
	 *
	 * @return bool|int
	 * @version 1.0.0
	 *
	 */
	public function Delete() {

		if ( empty( $this->ID ) ) {
			return false;
		}

		return VPP_DATABASE::Delete(
			$this->Table,
			VPP_DATABASE::Where(
				[
					'condition' => [
						[
							'column' => 'id',
							'value'  => $this->ID
						]
					]
				]
			)
		);

	}

	/**
	 * Video Metadata
	 *
	 * @return array|bool
	 * @version 1.0.0
	 *
	 */
	public function MetaData() {

		if ( empty( $this->Video ) ) {
			return false;
		}

		$tag = new VPP_VIDEO_TAG();
		$tag->Video_ID = $this->ID;
		$tags = $tag->SelectByVideoID();

		return [
			'type'        => 'video',
			'title'       => $this->Title,
			'description' => $this->Description,
			'video_url'   => wp_get_attachment_url( $this->Video ),
			'video_meta'  => wp_get_attachment_metadata( $this->Video ),
			'visibility'  => $this->Visibility,
			'user'        => ! empty( $this->User ) ? get_userdata( $this->User ) : null,
			'thumbnail'   => wp_get_attachment_image_url( $this->Thumbnail, 'full' ),
			'time'        => $this->Time,
			'modify'      => $this->Modify,
			'quality'     => [],
			'subtitle'    => [],
			'elements'    => [],
			'intro'       => null,
			'tags'        => ! empty( $tags ) ? $tags : explode( ' ', $this->Title ),
			'like_count'  => [],
		];

	}

	/**
	 * Get count all video
	 *
	 * @return mixed
	 * @version 1.0.0
	 *
	 */
	public function Count() {

		return VPP_DATABASE::Select(
			$this->Table,
			[
				'column' => [
					[
						'column' => 'COUNT(*)',
						'as'     => 'count'
					]
				]
			]
		)[0]['count'];

	}

	/**
	 * Set column on object variable
	 *
	 * @param $video
	 *
	 * @version 1.0.0
	 *
	 */
	private function SetVariable( $video ) {

		$this->ID = $video['id'];
		$this->Title = $video['title'];
		$this->Description = $video['description'];
		$this->Thumbnail = $video['thumbnail'];
		$this->Video = $video['video'];
		$this->Visibility = $video['visibility'];
		$this->User = $video['user'];
		$this->Time = $video['time'];
		$this->Modify = $video['modify'];

	}

}