<?php


class VPP_SKIN_YOUTUBE {

	/**
	 * YTPL_SKIN_YOUTUBE
	 *
	 * Basic object for skins. All events are inheritable,
	 * but it is recommended that you make changes to the appearance.
	 *
	 * Added from version 1.0.0
	 * Player, Playlist, Comments
	 *
	 */

	/**
	 * Traffic type
	 * @var null
	 */
	public $TrafficType = null;

	/**
	 * Traffic data
	 * @var null
	 */
	public $TrafficData = null;

	/**
	 * Type player `ytpl` Or `media`
	 * @var string
	 */
	protected $Type = 'vpp';

	/**
	 * ID video, playlist or media
	 * @var null
	 */
	protected $ID = null;

	/**
	 * Config data
	 * @var array
	 */
	protected $Config = [];

	/**
	 * YTPL_SKIN_YOUTUBE constructor.
	 *
	 * @param string $type
	 * @param null   $id
	 * @param array  $config
	 *
	 */
	public function __construct( $type = 'vpp', $id = null, $config = [] ) {

		$this->ID = $id;
		$this->Type = $type;
		$this->Config = $config;

	}

	/**
	 * Get name
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _get_name() {
		return 'Youtube Skin';
	}

	/**
	 * Get slug
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _get_slug() {
		return 'youtube';
	}

	/**
	 * Get Type
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _get_type() {
		return $this->Type;
	}

	/**
	 * Get id
	 *
	 * @return null
	 * @version 1.0.0
	 *
	 */
	public function _get_id() {
		return $this->ID;
	}

	/**
	 * Get setting video
	 *
	 * @return array
	 * @version 1.0.0
	 *
	 */
	public function _get_setting_video() {

		/** Custom Media */
		if ( $this->Type === 'media' ) {

			return [
				'type'        => $this->Type,
				'title'       => isset( $this->Config['title'] ) ? $this->Config['title'] : null,
				'description' => isset( $this->Config['description'] ) ? $this->Config['description'] : null,
				'video_url'   => isset( $this->Config['video_url'] ) ? $this->Config['video_url'] : null,
				'video_meta'  => isset( $this->Config['video_meta'] ) ? $this->Config['video_meta'] : null,
				'visibility'  => isset( $this->Config['visibility'] ) ? $this->Config['visibility'] : 'public',
				'user'        => null,
				'thumbnail'   => isset( $this->Config['thumbnail'] ) ? wp_get_attachment_image_url( $this->Config['thumbnail'], 'full' ) : null,
				'time'        => 0,
				'modify'      => 0,
				'quality'     => isset( $this->Config['quality'] ) ? $this->Config['quality'] : null,
				'subtitle'    => isset( $this->Config['subtitle'] ) ? $this->Config['subtitle'] : null,
				'element'     => null,
				'intro'       => null,
				'tags'         => isset( $this->Config['title'] ) ? explode( ' ', $this->Config['title'] ) : null,
				'like_count'  => [],
			];

		}

		/**
		 * YTPL Video
		 * @var  $video
		 */
		$video = new VPP_VIDEO( $this->ID );

		return $video->MetaData();

	}

	/**
	 * Before wrapper
	 *
	 * @param array $_class
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _before_wrapper( $_class = [] ) {

		return '<div class="ytpl-render ytpl-skin-' . $this->_get_slug() . ' ' . join( ' ', $_class ) . '" id="' . $this->_uniq_id() . '" style="display:none">';

	}

	/**
	 * After wrapper
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _after_wrapper() {
		return '</div>';
	}

	/**
	 * Enqueue style
	 *
	 * @version 1.0.0
	 *
	 */
	public function _enqueue_style() {

		wp_enqueue_style( 'vpp_skin_youtube_style', VPP_PLUGIN_CSS . 'template/vpp-skin-youtube-style.css' );

		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'vpp_skin_youtube_script', VPP_PLUGIN_JS . 'template/vpp-skin-youtube-script.js' );

		$localize = [
			'AjaxUrl' => admin_url( 'admin-ajax.php' ),
			'Nonce'   => wp_create_nonce( 'vpp' ),
			'SiteUrl' => get_site_url(),
		];
		wp_localize_script( 'vpp_skin_youtube_script', 'VPP_LOCALIZE', $localize );

	}

	/**
	 * Get uniq id
	 *
	 * @return string
	 * @version 1.0.0
	 *
	 */
	public function _uniq_id() {
		return uniqid( 'ytpl_' );
	}

	/**
	 * Player
	 *
	 * @return mixed
	 * @version 1.0.0
	 *
	 */
	public function Player() {

		ob_start();

		$this->_enqueue_style();
		$setting = $this->_get_setting_video();

		/** TODO Check exist video and check access */
		if ( ! $setting || ( $setting['visibility'] === 'private' && ! is_user_logged_in() ) ):
			if ( VPP::$setting->General_Alert() == 'yes' ):?>

                <div class="ytpl-access-denied">
                    <div class="ytpl-icon">
                        <div></div>
                    </div>
                    <div class="ytpl-description">
						<?php echo $setting ? __( 'Access Denied.' ) : __( 'Video has been deleted.' ); ?>
                    </div>
                </div>

			<?php endif;

			return ob_get_clean();
		endif;

		/** TODO Variables */
		$config = json_encode( [
			'id'           => $this->_get_id(),
			'src'          => $setting['video_url'],
			'quality'      => [],
			'subtitle'     => [],
			'suggestion'   => [],
			'carts'        => [],
			'end_screen'   => [],
			'ajax_disable' => VPP::$setting->General_Ajax(),
		] );

		?>

        <div class="ytpl-player ytpl-thumbnail ytpl-overlay ytpl-controller ytpl-top-bar"
             data-config="<?php echo esc_attr( $config ); ?>">
            <div class="ytpl-video-player">

                <!-- TODO Video -->
                <video class="ytpl-video" poster="<?php echo esc_url( $setting['thumbnail'] ) ?>"
                       style="width: 100% !important;">
                    <source src="<?php echo esc_url( $setting['video_url'] ); ?>" type="video/mp4">
                </video>

                <!-- TODO Overlays -->
                <div class="ytpl-top-overlay"></div>
                <div class="ytpl-bottom-overlay"></div>
                <div class="ytpl-dark-overlay"></div>

                <!-- TODO Thumbnail -->
                <div class="ytpl-thumbnail"
                     style="background-image: url(<?php echo esc_url( $setting['thumbnail'] ) ?>)"></div>

                <!-- TODO Center action - icon play -->
                <div class="ytpl-center-action">

                    <!-- <div class="ytpl-button ytpl-play"><?php echo VPP_ICON::$play; ?></div>
                    <div class="ytpl-button ytpl-pause"><?php echo VPP_ICON::$pause; ?></div> -->

                    <div class="ytpl-original-button">
                        <svg viewBox="0 0 68 48">
                            <path class="shape"
                                  d="M66.52,7.74c-0.78-2.93-2.49-5.41-5.42-6.19C55.79,.13,34,0,34,0S12.21,.13,6.9,1.55 C3.97,2.33,2.27,4.81,1.48,7.74C0.06,13.05,0,24,0,24s0.06,10.95,1.48,16.26c0.78,2.93,2.49,5.41,5.42,6.19 C12.21,47.87,34,48,34,48s21.79-0.13,27.1-1.55c2.93-0.78,4.64-3.26,5.42-6.19C67.94,34.95,68,24,68,24S67.94,13.05,66.52,7.74z"
                                  fill="#f00"></path>
                            <path class="icon" d="M 45,24 27,14 27,34" fill="#fff"></path>
                        </svg>
                    </div>

                </div>

                <!-- TODO Top bar -->
                <div class="ytpl-top-bar">
                    <div class="ytpl-top-bar-content">
                        <div class="ytpl-title"><?php echo esc_html( $setting['title'] ); ?></div>
						<?php if ( VPP::$setting->Default_Player_Description() == 'yes' ): ?>
                            <div class="ytpl-description-btn"><?php echo __( 'Description', VPP_TEXT_DOMAIN ); ?></div>
						<?php endif; ?>
                    </div>
                    <div class="ytpl-top-bar-button">
						<?php

						// Info button
						if ( VPP::$setting->Default_Player_Info() == 'yes' ): ?>
                            <div class="ytpl-item info">
								<?php
								echo VPP_ICON::$info;
								echo '<span>' . __( 'Info', VPP_TEXT_DOMAIN ) . '</span>'
								?>
                            </div>
						<?php endif;

						?>
                    </div>
                </div>

                <!-- TODO Controller -->
                <div class="ytpl-controller">
                    <div class="ytpl-progress">
                        <div class="ytpl-preview">
                            <div class="ytpl-video-frame">
                                <canvas class="ytpl-image"></canvas>
                                <span>00:00</span>
                            </div>
                        </div>
                        <div class="ytpl-progress-container">
                            <input type="range" min="0" max="100">
                            <div class="ytpl-preload" style="width: 0"></div>
                            <div class="ytpl-progressed" style="width: 0"></div>
                            <div class="ytpl-point-hover" style="left: 0"></div>
                            <div class="ytpl-intro">

                            </div>
                        </div>
                    </div>
                    <div class="ytpl-buttons">
                        <div class="ytpl-left-controller">
                            <div class="ytpl-button ytpl-play">
								<?php echo VPP_ICON::$play; ?>
                            </div>
                            <div class="ytpl-button ytpl-pause">
								<?php echo VPP_ICON::$pause; ?>
                            </div>
                            <div class="ytpl-button ytpl-volume">
								<?php echo VPP_ICON::$volume; ?>
                            </div>
                            <div class="ytpl-volume-process">
                                <div class="ytpl-process-container">
                                    <input type="range" value="100">
                                    <div class="ytpl-point-hover" style="left: 100%"></div>
                                    <div class="ytpl-processed" style="width: 100%"></div>
                                </div>
                            </div>
                            <div class="ytpl-time">
                                <span class="ytpl-current">00:00</span>
                                /
                                <span class="ytpl-duration"><?php echo esc_html( $setting['video_meta']['length_formatted'] ) ?></span>
                            </div>
                        </div>
                        <div class="ytpl-right-controller">
							<?php

							// Setting button
							if ( VPP::$setting->Default_Player_Setting() == 'yes' ): ?>
                                <div class="ytpl-button ytpl-setting">
									<?php echo VPP_ICON::$setting_player; ?>
                                </div>
							<?php endif;

							// Full screen button
							if ( VPP::$setting->Default_Player_Full_Screen() == 'yes' ): ?>
                                <div class="ytpl-button ytpl-full-screen">
									<?php echo VPP_ICON::$full_screen; ?>
                                </div>
							<?php endif;

							?>
                        </div>
                    </div>
					<?php

					// Popup setting
					if ( VPP::$setting->Default_Player_Setting() == 'yes' ): ?>

                        <div class="ytpl-setting-popup setting">
                            <ul>
								<?php if ( false ): ?>

                                    <li>
                                        <div><?php echo __( 'AutoPlay', VPP_TEXT_DOMAIN ); ?></div>
                                        <div></div>
                                    </li>

								<?php endif; ?>
                                <li class="action" data-action="speed">
                                    <div><?php echo __( 'Speed', VPP_TEXT_DOMAIN ); ?></div>
                                    <div>
                                        <span><?php echo __( 'Normal', VPP_TEXT_DOMAIN ); ?></span>
										<?php echo VPP_ICON::$nav_next; ?>
                                    </div>
                                </li>
                            </ul>
                        </div>

                        <div class="ytpl-setting-popup speed">
                            <div class="ytpl-back">
								<?php echo VPP_ICON::$nav_prev; ?>
                                <span><?php echo __( 'Back', VPP_TEXT_DOMAIN ); ?></span>
                            </div>
                            <ul>
                                <li value="0.25"><?php echo __( '0.25', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="0.5"><?php echo __( '0.5', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="0.75"><?php echo __( '0.75', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="1" class="active"><?php echo __( 'Normal', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="1.25"><?php echo __( '1.25', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="1.5"><?php echo __( '1.5', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="1.75"><?php echo __( '1.75', VPP_TEXT_DOMAIN ); ?></li>
                                <li value="2"><?php echo __( '2', VPP_TEXT_DOMAIN ); ?></li>
                            </ul>
                        </div>

					<?php endif; ?>
                </div>

                <!-- TODO Watermark -->
				<?php if ( VPP::$setting->Default_Player_Watermark() == 'yes' ): ?>
                    <div class="ytpl-watermark">
                        <div class="ytpl-image"
                             style="background:url(<?php echo esc_url( get_avatar_url( ! empty( $setting['user'] ) ? $setting['user']->data->ID : null ) ); ?>)"></div>
                        <div class="ytpl-box">
                            <div class="ytpl-title"><?php echo esc_html( ! empty( $setting['user'] ) ? $setting['user']->data->display_name : __( 'User', VPP_TEXT_DOMAIN ) ); ?></div>
                            <a href="<?php echo esc_url( get_author_posts_url( ! empty( $setting['user'] ) ? $setting['user']->data->ID : null ) ); ?>">
								<?php echo __( 'View User', VPP_TEXT_DOMAIN ); ?>
                            </a>
                        </div>
                    </div>
				<?php endif; ?>

                <!-- TODO Description -->
				<?php if ( VPP::$setting->Default_Player_Description() == 'yes' ): ?>
                    <div class="ytpl-description">
						<?php echo nl2br( esc_html( $setting['description'] ) ); ?>
                    </div>
				<?php endif; ?>

                <!-- TODO Info -->
				<?php if ( VPP::$setting->Default_Player_Info() == 'yes' ): ?>
                    <div class="ytpl-info">
                        <button class="ytpl-close">
							<?php echo VPP_ICON::$x; ?>
                        </button>
                        <div class="ytpl-content">
                            <table>
                                <tr>
                                    <td><?php echo __( 'Video Url', VPP_TEXT_DOMAIN ); ?></td>
                                    <td><?php echo esc_html( $setting['video_url'] ); ?></td>
                                </tr>
                                <tr>
                                    <td><?php echo __( 'Video Type', VPP_TEXT_DOMAIN ); ?></td>
                                    <td><?php echo esc_html( $setting['video_meta']['mime_type'] ); ?></td>
                                </tr>
                                <tr>
                                    <td><?php echo __( 'Video Time', VPP_TEXT_DOMAIN ); ?></td>
                                    <td><?php echo esc_html( $setting['video_meta']['length_formatted'] ); ?></td>
                                </tr>
                                <tr>
                                    <td><?php echo __( 'Video Resolution', VPP_TEXT_DOMAIN ); ?></td>
                                    <td><?php echo esc_html( $setting['video_meta']['width'] . 'px - ' . $setting['video_meta']['height'] . 'px' ); ?></td>
                                </tr>
                                <tr>
                                    <td><?php echo __( 'Video Size', VPP_TEXT_DOMAIN ); ?></td>
                                    <td><?php echo esc_html( $setting['video_meta']['filesize'] . 'kb' ); ?></td>
                                </tr>
                            </table>
                        </div>
                    </div>
				<?php endif; ?>

                <!-- TODO Context menu -->
				<?php if ( VPP::$setting->Default_Player_Enable_Right_Click() == 'yes' ): ?>
                    <div class="ytpl-context-menu">
                        <ul>
							<?php

							// Loop
							if ( VPP::$setting->Default_Player_Loop() == 'yes' ): ?>
                                <li data-type="loop">
									<?php echo VPP_ICON::$loop . __( 'Loop', VPP_TEXT_DOMAIN ) ?>
                                </li>
							<?php endif;

							// Video Info
							if ( VPP::$setting->Default_Player_Video_Info() == 'yes' ): ?>
                                <li data-type="video_info">
									<?php echo VPP_ICON::$info_2 . __( 'Video Info', VPP_TEXT_DOMAIN ) ?>
                                </li>
							<?php endif;

							?>
                        </ul>
                    </div>
				<?php endif; ?>

            </div>
        </div>

		<?php

		return ob_get_clean();

	}

}