<?php
/**
 * Plugin Name: Video Player Pro
 * Description: Video player pro plugin allows you to have a professional player with powerful tools. This plugin gives you a professional YouTube player skin. Tools: (Create playlists, manage views and control incoming traffic, create comments for videos, add multiple quality uses for video, Srt compatibility, embedding and more).
 * Author: IdeaLand
 * Version: 1.0.4
 * Text Domain: video-player-pro
 * Author URI: https://idea-land.co/
 */

/** SECURITY */
defined( 'ABSPATH' ) || exit();

class VPP {

	/** @var string $version plugin version */
	public static $version = '1.0.4';

	/** @var string $db_version database version */
	public static $db_version = '1.0.0';

	/** @var VPP_SETTING $setting setting */
	public static $setting = null;

	/** @var string $ajax_url ajax url */
	public static $ajax_url = null;

	/** @var string $nonce vpp */
	public static $nonce = 'vpp';

	/**
	 * VPP_LITE constructor.
	 */
	public function __construct() {

		include_once 'include/class/VPP_DATABASE.php';
		include_once 'include/VPP_REGISTER_PLUGIN.php';
		include_once 'include/class/VPP_SETTING.php';

		/** do_action init */
		do_action( 'vpp_init' );

		add_action( 'plugins_loaded', array( $this, 'PLUGINS_LOADED' ), 10 );
		add_action( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'LINK_PRO_VERSION' ) );
		add_filter( 'plugin_row_meta', array( $this, 'LINK_DOCUMENT' ), 10, 2 );

	}

	/**
	 * Run plugin
	 *
	 * @return false|void
	 * @version 1.0.0
	 *
	 */
	public function PLUGINS_LOADED() {

		define( 'VPP_TEXT_DOMAIN', 'video-player-pro' );
		define( 'VPP_PLUGIN_PATH', trailingslashit( plugin_dir_path( __FILE__ ) ) );
		define( 'VPP_PLUGIN_URL', trailingslashit( plugin_dir_url( __FILE__ ) ) );
		define( 'VPP_PLUGIN_CSS', VPP_PLUGIN_URL . 'assets/css/' );
		define( 'VPP_PLUGIN_IMG', VPP_PLUGIN_URL . 'assets/img/' );
		define( 'VPP_PLUGIN_JS', VPP_PLUGIN_URL . 'assets/js/' );

		self::$setting = new VPP_SETTING();
		self::$ajax_url = admin_url( 'admin-ajax.php' );

		/** do_action load */
		do_action( 'vpp_load' );

		include_once 'include/class/VPP_ICON.php';
		include_once 'include/class/VPP_VIDEO.php';
		include_once 'include/class/VPP_VIDEO_TAG.php';

		include_once 'include/VPP_STUDIO.php';
		include_once 'include/VPP_APP.php';

		/** do_action loaded */
		do_action( 'vpp_loaded' );

		load_plugin_textdomain(VPP_TEXT_DOMAIN , false, dirname( plugin_basename( __FILE__ ) ) . '/languages' ); 

	}

	/**
	 * Pro version link in plugin page
	 *
	 * @param $links
	 *
	 * @return mixed
	 * @version 1.0.0
	 *
	 */
	public function LINK_PRO_VERSION( $links ) {

		$settings_link = '<a href="' . esc_url( 'https://idea-land.co/video-player-pro/' ) . '" target="_blank" style="font-weight: bold; color:#ff008f;">' . __( 'Pro Plus Version', VPP_TEXT_DOMAIN ) . '</a>';
		array_unshift( $links, $settings_link );

		return $links;

	}

	/**
	 * Link document in plugin page
	 *
	 * @param $links
	 * @param $file
	 *
	 * @return array
	 * @version 1.0.0
	 *
	 */
	public function LINK_DOCUMENT( $links, $file ) {

		if ( plugin_basename( __FILE__ ) == $file ) {
			$row_meta = array(
				'docs' => '<a href="' . esc_url( 'https://idea-land.co/video-player-pro-documentation/' ) . '" target="_blank" style="font-weight:bold; ">' . esc_html__( 'Document', 'domain' ) . '</a>',
			);

			return array_merge( $links, $row_meta );
		}

		return (array) $links;

	}

}

new VPP();